@@ -13,6 +13,8 @@ use Carp;
 use Scalar::Util qw(looks_like_number);
 use URI::Escape;
 use Text::ParseWords;
+use JSON;
+
 our $AUTOLOAD;
 
 =head1 NAME
@@ -206,7 +208,9 @@ sub _null_safe_value {
     my $self = shift;
     my $value = shift;
     if (defined($value)) {
-        if (ref($value) && $self->string) {
+        if (JSON::is_bool($value)) {
+            return "$value";
+        } elsif (ref($value) && $self->string) {
             # We can deal with complex values withing string comparison
             if (ref($value) eq "ARRAY") {
                 return join ",",@{$value};
@@ -269,7 +273,7 @@ sub _delta_value {
 
 sub _switch_on_history {
     my ($self,$orig_request,$target) = @_;
-    my ($mbean,$operation) = ("jmx4perl:type=Config","setHistoryEntriesForAttribute");
+    my ($mbean,$operation) = ("jolokia:type=Config","setHistoryEntriesForAttribute");
     # Set history to 1 (we need only the last)
     return new JMX::Jmx4Perl::Request
       (EXEC,$mbean,$operation,
@@ -291,6 +295,7 @@ sub _base_value {
     }
     my $resp = shift @{$responses};
     my $req = shift @{$requests};
+    $np->nagios_die($resp->{error}) if $resp->{error};
     return $self->_extract_value($req,$resp);
 }
 
@@ -325,11 +330,11 @@ sub _normalize_value {
             next unless $units->[$i] eq $unit;
             my $ret = $value;
             my $u = $unit;
-            if ($ret > 1) {
+            if (abs($ret) > 1) {
                 # Go up the scale ...
                 return ($value,$unit) if $i == $#{$units};
                 for my $j ($i+1 .. $#{$units}) {
-                    if ($ret / $UNITS{$units->[$j]} >= 1) {                    
+                    if (abs($ret / $UNITS{$units->[$j]}) >= 1) {                    
                         $ret /= $UNITS{$units->[$j]};
                         $u = $units->[$j];
                     } else {
@@ -365,7 +370,7 @@ sub _verify_response {
         $np->nagios_die("Error: ".$resp->status." ".$resp->error_text.$extra);
     }
     
-    if (!$req->is_mbean_pattern && (ref($resp->value) && !$self->string)) { 
+    if (!$req->is_mbean_pattern && (ref($resp->value) && !$self->string) && !JSON::is_bool($resp->value)) { 
         $np->nagios_die("Response value is a " . ref($resp->value) .
                         ", not a plain value. Did you forget a --path parameter ?". " Value: " . 
                         Dumper($resp->value));
@@ -604,7 +609,8 @@ sub _format_label {
                 }
             } elsif ($what eq "t") {
                 my $code = $args->{code};
-                $ret .= sprintf $format . "s",$code == CRITICAL ? $self->critical : ($code == WARNING ? $self->warning : "");
+                my $val = $code == CRITICAL ? $self->critical : ($code == WARNING ? $self->warning : "");
+                $ret .= sprintf $format . "s",defined($val) ? $val : "";
             } elsif ($what eq "c") {
                 $ret .= sprintf $format . "s",$STATUS_TEXT{$args->{code}};
             } elsif ($what eq "n") {
